#!/usr/bin/env perl
# -*-CPerl-*-
# Last changed Time-stamp: <2019-02-16 14:06:05 mtw>

use Getopt::Long;
use Pod::Usage;
use Data::Dumper;
use File::Basename;
use File::Temp qw(tempfile);
use strict;
use warnings;
use Carp;
use vars qw/
    $T0 $T8 $TX $P0 $TINC $TEMP $EQ @FILES $TREEKIN %ENV $RATESUFFIX $RECOVER
/;

# defaults for global(s)
# $TREEKIN = "$ENV{HOME}/C/treekin/treekin";
$TREEKIN = 'treekin';
$T0 = 0.1;
$T8 = 10;
$TX = -1;
$TINC = 1.02;
$P0 = "";
$EQ = 0;
$TEMP = 37.0;
$RATESUFFIX="rates.bin";
my $outfile=undef;
my $OUT;

Getopt::Long::config('no_ignore_case');
pod2usage(-verbose => 1)
    unless GetOptions("t0=f"  => \$T0,
		      "t8=f"    => \$T8,
		      "tX=f"    => \$TX,
		      "inc=f"   => \$TINC,
		      "eq=i"    => \$EQ,
		      "T=f"     => \$TEMP,
		      "p0=s"    => \&process_opt_p0,
		      "rs=s"    => \$RATESUFFIX,
		      "o=s"     => \$outfile,
		      "man"     => sub{pod2usage(-verbose => 2)},
		      "help"    => sub{pod2usage(-verbose => 1)},
		      "recover" => \$RECOVER,
		   );

defined $outfile ? open($OUT, ">", $outfile) : open($OUT, ">&STDOUT");
die $! unless (defined $OUT);

my ($rs, $cs)  = parse_bar_map();
my $p_zeros    = $P0;
my $start_time = $T0;
my $stop_time  = $T8; 
my $global_time = 0.0;
#print Dumper(\@FILES);

# main loop
my $flag = 0;
for (my $file = 0; $file<=$#FILES; $file++) {
  $stop_time = $TX, $flag = 1 if ($#FILES == $file) && ($TX > 0);
  print $OUT "# Rates file: ".$FILES[$file]." (#$file)\n";
  if ($EQ > 0){
    if ($file == $#FILES){
      $stop_time = $EQ;
      print $OUT "# Info: last landscape, setting t8 to $EQ\n";
    }
  }
#  print $OUT "p_zeros $p_zeros\n";
  my $command = build_command($p_zeros,
			      $stop_time,
			      $FILES[$file],
			      $RECOVER,
			      $RATESUFFIX,
			    );
  print $OUT "# Cmd: $command\n";
  my ($stop, $densities, $tc) = do_simulation($command, $flag);
  if ($stop == -1) {
    print STDERR "Treekin ERROR at inputfile $FILES[$file]\n";
    exit 1;
  }
  # dump time course to STDOUT
  dump_time_course_new($stop, $global_time, $tc, $flag);
  $global_time += $stop;

  last if $file == $#FILES;
  #print ">> file is $file<<\n";
  $p_zeros = remap_densities($densities, $cs->[$file], $cs->[$file+1]);
}
unlink qw(evecs.bin evals.bin) if $RECOVER;         # clean up

#--
# Returns the max length of a command line on this systems as queried using
# getconf. If that fails, use a default of 32000 (as used e.g. by Cygwin). On
# common Linux systems, it is much higher (130k to 230k).
sub get_max_cmd_length {
    my $default = 128_000;

    # TODO The getconf value is for some reasons too high :-( But only when
    # Perl is calling the shell. Manual call works... ?! Use something lower.
    # return $default;

    # Get error code for "cmd not found"
    system('@@@@@ 2>/dev/null');
    my $cmd_not_found_err = $? or die 'This should have failed, bug...';

    # Since retrieving the maximum arg length directly fails, use an iterative
    # approach: extend a gibberish test command until the system call fails
    # because of a too long arg list.
    my $limit_inc = 10_000;         # extend test cmd by this amount of chars
    my $long_test_cmd;
    while(length($long_test_cmd) <= 235_000) {  # 235k suffices for 10k states
      $long_test_cmd .= "@" x $limit_inc;
      # TODO This prints out a nasty error message when arg list is too long.
      system($long_test_cmd . " 2>/dev/null");
      my $exit_code = $?;
      die 'This should have failed, bug...' if $exit_code == 0;
      last if $exit_code != $cmd_not_found_err;         # arg list too long
    }
    my $limit = length($long_test_cmd) - $limit_inc;    # previous one worked

    # Check it worked out.
    if ($limit <= 0) {
      print STDERR '# Could not determine max command line length',
                   ", using default of $default";
      return $default;
    }

    # my $limit   = `getconf ARG_MAX`;

    # if (not $? or not defined $limit or $limit <= 0) {
    #     $limit = $default;
    #     print STDERR '# Could not retrieve max command line length from ',
    #                  "getconf, using default of $default";
    # }

    # # Account for contents of env vars as well as (up to) 8 byte variable
    # # pointers of these.
    # my $substract = `env | wc -c` - `env | wc -l` * 8;
    # $substract = 0 if not $? or not defined $substract or $substract <= 0;

    # $limit -= $substract;

    return $limit;
}

#--
sub get_max_pop_switch_count {
    # Get max length of a command line call on this system.
    my $max_cmd_len = get_max_cmd_length;

    # Get max switch length from example.
    my $max_switch_len   = length ' --p0 9999=1.234567e-89';

    # Leave some 1000 chars over for the rest of the command and substract
    # 2048 as suggested by POSIX to "let the process modify its args".
    my $max_switch_count = int( ($max_cmd_len-3048) / $max_switch_len );

    return $max_switch_count;
}

#---
sub trim_pops_to_max_cmd_length {
    my ($pops_ref) = @_;
    # Determine max command length for population switches to use (only once
    # because it does several external call, needs Perl v5.10).
    # Leave some 1000 chars over for the rest of the command and substract
    # 2048 as suggested by POSIX to "let the process modify its args".
    use v5.10;
    use List::Util qw(sum);
    state $max_switch_length = get_max_cmd_length - 3048;
    die 'Max command length is too small or was incorrectly determined'
        if $max_switch_length < 100;    # allows for about 2--3 switches

    # Current length consists of 1) length of --p0 switches and = signs, 2)
    # length of state numbers (1..n), 3) digits of populations.
    my $current_switch_length = length build_p0_args($pops_ref);

    # Nothing to be done, use all switches.
    return if $current_switch_length <= $max_switch_length;

    ##########################################################################
    ##### Variant one: delete smallest populations first               #######
    ##########################################################################

    # Too many pops! Order populations (descending) and set lowest pops to
    # zero until we are short enough. Do not remove to preserve correct
    # indexing. Pops start at index 1!
    my @sorted_pop_indices
        = sort {$pops_ref->[$b] <=> $pops_ref->[$a]} 1..$#$pops_ref;

    # Drop indices of zero-pops since they are not printed anyway.
    pop @sorted_pop_indices while $pops_ref->[$sorted_pop_indices[-1]] == 0;

    # Remove lowest pops until length drops below allowed maximum.
    my ($removed_pop_count, $total_removed_pop);
    while ($current_switch_length > $max_switch_length) {
        # Reduce length by length of lowest population's switch
        my $lowest_pop_index = pop @sorted_pop_indices;
        my $lowest_pop_switch
            = build_p0_args($lowest_pop_index, $pops_ref->[$lowest_pop_index]);
        $current_switch_length -= length $lowest_pop_switch;
        $total_removed_pop += $pops_ref->[$lowest_pop_index];
        $pops_ref->[$lowest_pop_index] = 0;
        $removed_pop_count++;
    }

    # print STDERR "# nulling $removed_pop_count populations to reduce ",
    #              "population switch length to $current_switch_length\n",

    ##########################################################################
    ##### Variant two: delete populations from end of populations list #######
    ##########################################################################

    # # TODO Re-investigate this. Ergodicity is a property of the matrix, not of
    # # the distribution of populations. Nulling populations should NOT have an
    # # impact.

    # # Too many pops! Starting from the end of the array, set pops to zero
    # # until we are short enough (hope that matrix ergodicity is preserved!). Do not remove
    # # to preserve correct indexing. Pops start at index 1!
    # # Zero-pops are not printed.
    # my $removed_pop_count = 0;      # number of removed switches
    # my $total_removed_pop = 0;      # sum of total population removed
    # my $last_pop_index = @$pops_ref;
    # while (--$last_pop_index > 1) {
    #     my $last_pop = $pops_ref->[$last_pop_index];
    #     next if $last_pop == 0;            # already zero, continue
    #     my $last_pop_switch = build_p0_args($last_pop_index, $last_pop);

    #     # Reduce total switch length by length of last population's switch.
    #     $current_switch_length -= length $last_pop_switch;
    #     $pops_ref->[$last_pop_index] = 0;
    #     $total_removed_pop += $last_pop;
    #     $removed_pop_count++;

    #     last if $current_switch_length <= $max_switch_length;   # short enough
    # }

    ##########################################################################
    ##### End of variants                                              #######
    ##########################################################################

    $total_removed_pop = sprintf "%.2f%%", $total_removed_pop * 100;
    print STDERR "# nulling $removed_pop_count populations ",
                 "($total_removed_pop loss) to reduce population switch ",
                 "length to $current_switch_length\n",
                 ;

    return 1;                   # changed population array
}

#--
# sub trim_pops_to_max_cmd_length {
#     my ($pops_ref) = @_;
#     # Determine max number of population switches to use (only once because it
#     # does an external call, needs Perl v5.10).
#     use v5.10;
#     state $max_switch_count = get_max_pop_switch_count;
#
#     return if @$pops_ref <= $max_switch_count;      # savely use all pops
#
#     # Too many pops! Order populations and set lowest pops to zero. Do
#     # not remove to preserve correct indexing.
#     my @removed_indices = (             # sort idx descending by population
#         sort {$pops_ref->[$b] <=> $pops_ref->[$a]} 0..$#$pops_ref
#     ) [$max_switch_count..$#$pops_ref];     # remove idx max_switch_count->end
#
#     print STDERR '# nulling ', scalar(@removed_indices), ' populations to ',
#                  'reduce population switch count to ', $max_switch_count, "\n";
#     $pops_ref->[$_] = 0 foreach @removed_indices;   # zero out removed pops
# }

##---
sub dump_time_course_old {
  my ($stop, $global_time, $tc, $flag) = @_;

  # print time course
  foreach my $line (@$tc) {
    if ($line =~ m/^\#/) {
      print $OUT $line;
      next;
    }
    $line =~ s/^\s+//;
    my @values = split /\s+/, $line;
    my $time = shift @values;
    $time += $global_time;
    printf ($OUT "%22.20e ",$time);
    print $OUT join(" ", @values), "\n";
  }
  print $OUT "&\n";

  return if $flag;
  $stop += $global_time;                # FK: return value not used?!
}

#---
# This new version received cleanups and changes the way the the data is
# processed: only the time value is cleanly updated with the global time. The
# rest of the line remains untouched. This preserves e.g. the trailing space
# printed by Treekin at the end of each line.
# TODO replace old one
sub dump_time_course_new {                  # FK
  my ($stop, $global_time, $tc) = @_;

  # Regexp to match the time field of a data line.
  my $match_time = qr/^\s*([-+.eE\d]+)/;

  # Print time course.
  foreach my $line (@$tc) {
    if ($line =~ m/^\#/) {              # print header line as is
      print {$OUT} $line;
      next;
    }

    # Update the line's time stamp.
    $line =~ /$match_time/;             # extract time
    my $time = $1;
    $time = sprintf "%22.20e", $time + $global_time;   # update time
    $line =~ s/$match_time/$time/;      # update line
    print {$OUT} $line;
  }

  # Print record separator.
  print $OUT "&\n";
}

#---
sub calculate_stop_time {               # FK: sub never called?!
  my $start = shift;
  my $time;
  for ($time=$start; $time <= $start+$T8; $time *= $TINC) {;}
  return $time;
}

#---
sub build_p0_args {
  my ($pops_ref) = @_;

  # Allow passing a single index and population value. Return empty string if
  # population is zero.
  return $_[1] == 0 ? q{} : " --p0 $_[0]=$_[1]" if @_ == 2 and not ref $_[0];

  # Build command from passed population(s).
  my $p0 = "";
  # my $p0_sum = 0.0;
  for(my $i=1; $i<=$#$pops_ref; $i++) {
    my $pop = $pops_ref->[$i];
    next if ($pop == 0.);
    $p0 .= " --p0 $i=$pop";
    # $p0_sum += $pop;
  }

  return $p0;
}

#---
sub remap_densities {
  my ($densities, $current, $next) = @_;
  my $tuples = [ grep {!     (($_->[0] == -1)
			      || ($_->[1] == -1))} @{zip($current, $next)}];
  # there can be duplicate entries in the barmap file
  $tuples = uniq_tuples($tuples);
  #print STDERR ">>tuples\n";
  #print STDERR Dumper(\$tuples);
  #print STDERR ">>tuples DONE\n";
  my $dens_sum=0.;
  my @d = @$densities;
  shift @d;
  $dens_sum += $_ for @d;
  print STDERR "# we have ".eval($#$densities)." densities with a total density of $dens_sum from the previous landscape\n";

  # now get # of states in new landscape
  my $maxnew=0;
  for(my $i=0; $i<= $#$tuples; $i++) {
    my $new = $tuples->[$i]->[1];
    if($new > $maxnew){ $maxnew = $new }
  }

  my @p = (0) x ($maxnew);
  $maxnew=0;
  for(my $i=0; $i<= $#$tuples; $i++) {
    my $new = $tuples->[$i]->[1];
    my $old = $tuples->[$i]->[0];
    if ($old <= 0){
      print STDERR "skipping lmin $old\n";
      next;  # do we need a next at the end here?
    }
    $p[$new] += $densities->[$old];
    if($new > $maxnew){ $maxnew = $new }
  }

  # Throw away lowest population values to not exceed the max command line
  # length in the next Treekin call. Iterate because rescaling may change the
  # length of the p0 switches (from 23 to 27 chars).
  my $pops_changed = 1;
  while ($pops_changed) {
      my $total = 0.;
      $total += $_ for @p;
      print STDERR "# remapped! New total density is $total\n";

      # Re-scale population to a total of 100% when deviating too much.
      if (abs($total-1) >= 1e-4){       # delta old/new population.
          print STDERR "# rescaling total population to 1.\n";
          $_ /= $total foreach @p;
      }

      $pops_changed = trim_pops_to_max_cmd_length(\@p);
  }

  print STDERR "# max lmin in new landscape is $maxnew\n\n";

  #print STDERR "\@p:\n";
  #print STDERR Dumper(\@p);

  # my $yy = zip(number_sequence(), \@p);         # FK: unused variable?!
  my $p0 = build_p0_args(\@p);
  # my $p0 = "";
  # my $p0_sum = 0.0;
  # for(my $i=1; $i<= $#p; $i++) {
  #   my $pop = $p[$i];
  #   next if ($pop == 0.);
  #   $p0 .= " --p0 $i=$pop";
  #     $p0_sum += $pop;
  #   }

    # FK: We are rescaling to 100% anyway. No need to check this.
    # if ($p0_sum > 1.01) {
    #   print STDERR "$p0\np0_sum = $p0_sum\n";
    #   print STDERR Dumper([grep {!  (($_->[0] == -1)
    #   		        || ($_->[1] == -1))} @$tuples]);
    #   print STDERR Dumper(\$p0);
    #   die $!
    # }

    # if ($p0_sum < 0.99) {
    #   print STDERR "$p0\np0_sum = $p0_sum\n";
    #   print STDERR Dumper([grep {!  (($_->[0] == -1)
    #   		        || ($_->[1] == -1))} @$tuples]);
    #   print STDERR Dumper(\$p0);
    #   die $!
    # }

  return $p0;
}

#---
sub uniq_tuples {
  my $tuples = shift;
  my @uniq = ();
  my $h = {map { $_->[0] => $_->[1] } @$tuples};
  foreach my $key (sort {$a <=> $b} keys %$h) {
    push @uniq, [$key, $h->{$key}];
  }
  return \@uniq;
}

#---
sub make_eigenval_links {
  # Recovering from pre-computed eigenvalues. Since Treekin reads these from
  # hardcoded file names 'evecs.bin' and 'evals.bin', links need to be created
  # from the respective eigenvalue file of the current rate matrix to match
  # the hardcoded file names.
  my ($rates_file, $rate_suffix) = @_;
  my $match_suffix = quotemeta $rate_suffix;    # escape regex meta chars
  my $base_name    = $rates_file =~ s/[.]$match_suffix//r;  # remove suffix
  my $evecs_file   = "$base_name.evecs.bin";    # eigenvecs / values are
  my $evals_file   = "$base_name.evals.bin";    # stored along w/ rate mat
  die "Eigenvector file '$evecs_file' and eigenvalue file '$evals_file' ",
      'required when using --recover, but not present or empty'
    unless -s $evecs_file and -s $evals_file;

  # Create symlinks.
  unlink qw(evecs.bin evals.bin);               # delete if existing
  symlink $evecs_file, 'evecs.bin';
  symlink $evals_file, 'evals.bin';
}

#---
sub build_command {
  my ($p_zeros, $stop, $file, $recover, $ratesuffix) = @_;
  my $command = "$TREEKIN -m I";
  $command .= " --feps=-1.0";           # solves some numerical issues
  $command .= " --tinc=$TINC";
  $command .= " --t0=$T0";
  $command .= " --t8=$stop";
  $command .= " --Temp=$TEMP";
  $command .= " --bin";
  if ($recover) {
      $command .= " --recoverE";
      make_eigenval_links $file, $ratesuffix;
  }
  $command .= $p_zeros;
  $command .= " < $file";
 # print Dumper($command);
  return $command;
}

#---
sub do_simulation {
  my ($command, $flag) = @_;

  # Redirect STDERR of Treekin to a temporary file.
  my ($treekin_error_fh, $treekin_error_file)
    = tempfile 'barmap_treekin_err.tmp.XXXXX', UNLINK => 1;
  $command .= " 2> $treekin_error_file";

  # Run Treekin and capture output.
  print STDERR "$command\n\n";
  my @output = `$command`;

  # Capture treekin error.
  if ($#output == $[-1) {
    # Print STDERR from temp file.
    print STDERR "ERROR: Treekin failed. Log:\n", <$treekin_error_fh>;
    return (-1, undef, undef); 
  }

  # get last line of time course not beginning with a '#' (i.e. no comment)
  my $lastline_index = (grep {$output[$_] !~ /^#/} 0..$#output)[-1];
  die 'No data found in Treekin output' unless defined $lastline_index;
  $output[$lastline_index] = '#' . $output[$lastline_index] if $flag == 1;
  my $lastline = $output[$lastline_index];

  $lastline =~ s/^\s+//;
  # extract values from last line of time course
  my @values = split /\s+/, $lastline;

  # check that we found at least two and nothing but numbers.
  use List::Util qw(all);
  use Scalar::Util qw(looks_like_number);
  die 'Received invalid last output line from Treekin: ', $lastline
    unless @values > 1 and all {looks_like_number $_} @values;

  my @valuescp = @values;
  shift @valuescp;
  my $vsum=0.;
  $vsum += $_ for @valuescp;
  print STDERR "total population at the end of the simulation is $vsum\n";
  # get stop time
  my $stop = $values[0];
  return ($stop, \@values, \@output);
}

#---
sub filter { [map { $_[0]->($_) ? $_ : ()} @{$_[1]}] }

#---
#sub is_defined { return 1 if defined $_->[1]; return 0 }

#---
sub process_opt_p0 {
  my $string = $_[1];
  if ($string =~ m/:/) {
    my @p0s = split /:/, $string;
    for my $p0 (@p0s) {
      my ($lmin, $value) = split(/=/, $p0);
      $P0 .= " --p0 $lmin=$value";
    }
  }
  else {
    my ($lmin, $value) = split(/=/, $string);
    $P0 .= " --p0 $lmin=$value";
  }
}

#---
sub parse_bar_map {
  my (@rows, $columns,$fn);
  while (<>) {
    s/^\#//, @FILES = split, next if $. == 1;
    chomp;
    # smash line into substrings of length 3 and capture only the odd ones
    # trim whitespace and convert empty strings to undef values
    # mtw 20171031: read up to 4 digits in group 1, ie allow max 9999 lmins
    # this breaks backward compatibility !!!
    my $row = [map {s/ //g; $_ eq "" ? -1 : $_ } (m/(....)(?:...)?/g)];
    # memorize row in a lol
    push @rows , $row;
    $columns = to_columns($#$row) if $. == 2;
    # memorize columns in a lol
    $columns->(zip(number_sequence(), $row));
  }
  foreach (@FILES){
    if ($RATESUFFIX eq "rates.r.bin"){
      $fn = fileparse($_, qr/\.r\.bar/);
    }
    else{
      $fn = fileparse($_, qr/\.bar/);
    }
    $_ = "$fn.".$RATESUFFIX;
    }
  # return lol's of rows and columns
  # print Dumper(@FILES);die;
  # print Dumper(\@rows);
  #print Dumper($columns->());
  return (\@rows, $columns->());
}

#---
# zip two lists togather into a list of tuples
# [a1, a2, ...], [b1, b2, ...] -> [[a1, b1], [a2, b2], ...]
sub zip {
  my ($A, $B) = @_;

  $A = array_to_iterator($A) if ref($A) eq 'ARRAY';
  $B = array_to_iterator($B) if ref($B) eq 'ARRAY';
 # print STDERR ">>zip() A:\n";
 # print STDERR Dumper($A->());
 # print STDERR ">>zip(): A DONE\n";
  my @result;
  while (   defined(my $a = $A->())
	 && defined(my $b = $B->())) {
    push @result, [$a, $b];
  }
 # print STDERR ">>zip() result:\n";
 # print STDERR Dumper(\@result);
 # print STDERR ">>zip(): result DONE\n";
  return \@result;
}

#---
# unzip a list of tupels into a tuple of lists
# [[a1, b1], [a2, b2], ...] -> [[a1, a2, ..], [b1, b2, ...]]
sub unzip {
  my (@A, @B);
  push(@A, $_->[0]), push(@B, $_->[1]) for @{shift()};

  return [\@A, \@B];
}

#---
# (val1, val2, ...) | [val1, val2, ...] -> {}
# converts an array into an iterator
sub array_to_iterator {
  my @array = (ref $_[0] eq 'ARRAY') ? @{shift()} : @_;

  return sub { shift @array }
}

#---
# n -> { }
# lazy version of infinit list [n..] of integers starting at value n
# implemented as closure
sub number_sequence {
  my $value = defined $_[0] ? $_[0] : 0;

  return sub { $value++ }
}

#---
# incrementally build lol from list of duples, implemented as closure
# [[idx1,val1], [idx2,val2], ...] -> [[values, val1],[values, val2], ...]
sub to_columns {
  my $lol = [ map{[]} $[..$_[0] ]; # empty lol with right dimension
  return sub {
    return $lol unless defined $_[0]; # return result
    map {push @{$lol->[$_->[0]]}, $_->[1]} @{$_[0]}; 
  }
}

=head1 NAME

barmap_simulator.pl - perform a dynamic landscape simulation, e.g. for
co-transcriptional RNA folding

=head1 SYNOPSIS

barmap_simulator.pl [[-t0 I<FLOAT>] [-t8 I<FLOAT>] [-inc I<FLOAT>]
[-p0 I<STRING>] [-eq I<INT>] barmap-file

=head1 DESCRIPTION

This tool performs a BarMap simulation on a set of consecutive
landscapes. It expects a BarMap file that contains the exact landscape
mapping (pre-computed with I<bar_map.pl>) as last argument. Starting
from an initial population distribution on the first landscape (option
B<-p0>), this tool calls I<treekin> to simulate the folding dynamics
on each landscape for a certain time (measured in internal time,
option B<-t8>). At the end of such an increment, population of each
macrostate is mapped to the corrsponding macrostate in the next
landscape, and I<treekin> is called with this new start population.

A consistency check for overall popultion density is performed at
every elongation step. This ensures that no population density is lost
in the course of the simulation.

=head1 NOTES

I<treekin> simulations are performed on binary rates files per
default. This implies that for each C<.bar> file referenced in
C<barmap-file>, the corresponding binary rates file is present in the
current working directory.

=head1 OPTIONS

=over 4

=item B<-t0> I<FLOAT>

Set start time of kinetic simulation to I<FLOAT> (default: 0.1).

=item B<-t8> I<FLOAT>

Set stop time of kinetic simulation to I<FLOAT> (default: 10.0).

=item B<-inc> I<FLOAT>

Set time increment of kinetic simulation to I<FLOAT> (default: 1.02).

=item B<-T> I<FLOAT>

Set simulation temperature to I<FLOAT> (default: 37.0).

=item B<-p0> I<STRING>

Specify initial population for minima on the first energy
landscape. For example if we want to start with 35% of the population
in minimum 3 and 65% in minimum 5 the I<STRING> looks as follows
"3=.35:5=.65" (floats must sum to 1).

=item B<-eq> I<INT>

If speficied, treekin simulation on the last landscape is performed
until the pecified stop time (recommended value 1000000).

=item B<-rs> I<STRING>

Suffix of the binary rates files produces by barriers (default: rates.bin)

=item B<-recover>

Recover from pre-computed (binary) eigenvectors and values (e.g. computed in
parallel) as computed by Treekin's C<--dumpE> option.  Speeds up repeated
simulations with different stop times etc.  For input file C<foo.rates.bin>,
eigenvectors and -values need to be stored in files C<foo.evecs.bin> and
C<foo.evals.bin>.

=back

=head1 SOURCE AVAILABITY

Source code for this distribution is available from the
L<ViennaRNA/BarMap GitHub
repository|https://github.com/ViennaRNA/BarMap>.

=head1 AUTHORS

Christoph Flamm, Michael T. Wolfinger

=head1 BUGS

Please send comments and bug reports to E<lt>mtw@tbi.univie.ac.at<gt>.

=cut
    
__END__
# my $a1 = [qw/a b c d e f g/];
# my $b1 = [qw/h i j k l m n/];
# my $c1 = [qw/o p q r s t u/];

# my $a2 = zip_with_natural_numbers($a1, natural_numbers());
# my $b2 = zip_with_natural_numbers($b1, natural_numbers());
# my $c2 = zip_with_natural_numbers($c1, natural_numbers());

# print Dumper($a2);
# print Dumper($b2);

# my $cols = to_cols($#$a2);
# $cols->($a2);
# print Dumper($cols->());
# $cols->($b2);
# $cols->($c2);
# print Dumper($cols->());

#my $a3 = unzip($a2);
#print Dumper($a3);
